<?php
/**
 * В отличии от оригинального класса, сохраняет сырые данные в RawMemcached
 * без сериализации.
 *
 * @author Tokarchuk Andrey
 * @see http://tokarchuk.ru
 * @link netandreus@gmail.com
 *
 */

class ZendExtra_Cache_Backend_RawMemcached extends Zend_Cache_Backend_Memcached
{

    /**
     * Получаем сырые данные
     */
    public function load($id, $doNotTestCacheValidity = false)
    {
        $tmp = $this->_memcache->get($id);
        /*
        if (is_array($tmp) && isset($tmp[0])) {
            return $tmp[0];
        }*/
        if($tmp) return $tmp;
        return false;
    }

    /**
     * Проверка существования сырых данных
     */
    public function test($id)
    {
        $tmp = $this->_memcache->get($id);
        /*
        if (is_array($tmp)) {
            return $tmp[1];
        }
        */
        if($tmp)
            return $tmp;
        return false;
    }

    /**
     * Сохранение сырых данных без сериализации
     */
    public function save($data, $id, $tags = array(), $specificLifetime = false)
    {
        $lifetime = $this->getLifetime($specificLifetime);
        if ($this->_options['compression']) {
            $flag = MEMCACHE_COMPRESSED;
        } else {
            $flag = 0;
        }

        // ZF-8856: using set because add needs a second request if item already exists
        //$result = @$this->_memcache->set($id, array((string)$data));
        $result = @$this->_memcache->set($id, $data, $flag, $lifetime);
        if (count($tags) > 0) {
            $this->_log(self::TAGS_UNSUPPORTED_BY_SAVE_OF_MEMCACHED_BACKEND);
        }

        return $result;
    }


   /**
    * Т.к. lifetime теперь не хранится в data, то нельзя его узнать при get-запросе
    * к memcached, как следствие,  touch() работать не сможет.
    */
    public function touch($id, $extraLifetime)
    {
        /*
        if ($this->_options['compression']) {
            $flag = MEMCACHE_COMPRESSED;
        } else {
            $flag = 0;
        }
        $tmp = $this->_memcache->get($id);
        if (is_array($tmp)) {
            $data = $tmp[0];
            $mtime = $tmp[1];
            if (!isset($tmp[2])) {
                // because this record is only with 1.7 release
                // if old cache records are still there...
                return false;
            }
            $lifetime = $tmp[2];
            $newLifetime = $lifetime - (time() - $mtime) + $extraLifetime;
            if ($newLifetime <=0) {
                return false;
            }
            // #ZF-5702 : we try replace() first becase set() seems to be slower
            if (!($result = $this->_memcache->replace($id, array($data, time(), $newLifetime), $flag, $newLifetime))) {
                $result = $this->_memcache->set($id, array($data, time(), $newLifetime), $flag, $newLifetime);
            }
            return $result;
        }
        */
        return false;
    }
}

?>